@@ -1,13 +1,13 @@
 import logging
+from urllib.parse import quote
 
 from django import http
+from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth import login
 from django.shortcuts import redirect
 from django.urls import reverse, reverse_lazy
-from django.utils import six
-from django.utils.http import urlquote
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.views import generic
 
 from oscar.core.loading import get_class, get_classes, get_model
@@ -16,10 +16,8 @@ from . import signals
 
 ShippingAddressForm, ShippingMethodForm, GatewayForm \
     = get_classes('checkout.forms', ['ShippingAddressForm', 'ShippingMethodForm', 'GatewayForm'])
-OrderCreator = get_class('order.utils', 'OrderCreator')
 UserAddressForm = get_class('address.forms', 'UserAddressForm')
 Repository = get_class('shipping.repository', 'Repository')
-AccountAuthView = get_class('customer.views', 'AccountAuthView')
 RedirectRequired, UnableToTakePayment, PaymentError \
     = get_classes('payment.exceptions', ['RedirectRequired',
                                          'UnableToTakePayment',
@@ -30,14 +28,8 @@ CheckoutSessionMixin = get_class('checkout.session', 'CheckoutSessionMixin')
 NoShippingRequired = get_class('shipping.methods', 'NoShippingRequired')
 Order = get_model('order', 'Order')
 ShippingAddress = get_model('order', 'ShippingAddress')
-CommunicationEvent = get_model('order', 'CommunicationEvent')
-PaymentEventType = get_model('order', 'PaymentEventType')
-PaymentEvent = get_model('order', 'PaymentEvent')
 UserAddress = get_model('address', 'UserAddress')
-Basket = get_model('basket', 'Basket')
-Email = get_model('customer', 'Email')
 Country = get_model('address', 'Country')
-CommunicationEventType = get_model('customer', 'CommunicationEventType')
 
 # Standard logger for checkout events
 logger = logging.getLogger('oscar.checkout')
@@ -48,7 +40,7 @@ class IndexView(CheckoutSessionMixin, generic.FormView):
     First page of the checkout.  We prompt user to either sign in, or
     to proceed as a guest (where we still collect their email address).
     """
-    template_name = 'checkout/gateway.html'
+    template_name = 'oscar/checkout/gateway.html'
     form_class = GatewayForm
     success_url = reverse_lazy('checkout:shipping-address')
     pre_conditions = [
@@ -64,10 +56,10 @@ class IndexView(CheckoutSessionMixin, generic.FormView):
             signals.start_checkout.send_robust(
                 sender=self, request=request)
             return self.get_success_response()
-        return super(IndexView, self).get(request, *args, **kwargs)
+        return super().get(request, *args, **kwargs)
 
     def get_form_kwargs(self):
-        kwargs = super(IndexView, self).get_form_kwargs()
+        kwargs = super().get_form_kwargs()
         email = self.checkout_session.get_guest_email()
         if email:
             kwargs['initial'] = {
@@ -93,7 +85,7 @@ class IndexView(CheckoutSessionMixin, generic.FormView):
                 self.success_url = "%s?next=%s&email=%s" % (
                     reverse('customer:register'),
                     reverse('checkout:shipping-address'),
-                    urlquote(email)
+                    quote(email)
                 )
         else:
             user = form.get_user()
@@ -129,7 +121,7 @@ class ShippingAddressView(CheckoutSessionMixin, generic.FormView):
     saved in the session and later saved as ShippingAddress model when the
     order is successfully submitted.
     """
-    template_name = 'checkout/shipping_address.html'
+    template_name = 'oscar/checkout/shipping_address.html'
     form_class = ShippingAddressForm
     success_url = reverse_lazy('checkout:shipping-method')
     pre_conditions = ['check_basket_is_not_empty',
@@ -153,7 +145,7 @@ class ShippingAddressView(CheckoutSessionMixin, generic.FormView):
         return initial
 
     def get_context_data(self, **kwargs):
-        ctx = super(ShippingAddressView, self).get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
         if self.request.user.is_authenticated:
             # Look up address book data
             ctx['addresses'] = self.get_available_addresses()
@@ -168,7 +160,7 @@ class ShippingAddressView(CheckoutSessionMixin, generic.FormView):
             '-is_default_for_shipping')
 
     def post(self, request, *args, **kwargs):
-        # Check if a shipping address was selected directly (eg no form was
+        # Check if a shipping address was selected directly (e.g. no form was
         # filled in)
         if self.request.user.is_authenticated \
                 and 'address_id' in self.request.POST:
@@ -182,7 +174,7 @@ class ShippingAddressView(CheckoutSessionMixin, generic.FormView):
             else:
                 return http.HttpResponseBadRequest()
         else:
-            return super(ShippingAddressView, self).post(
+            return super().post(
                 request, *args, **kwargs)
 
     def form_valid(self, form):
@@ -191,14 +183,14 @@ class ShippingAddressView(CheckoutSessionMixin, generic.FormView):
             (k, v) for (k, v) in form.instance.__dict__.items()
             if not k.startswith('_'))
         self.checkout_session.ship_to_new_address(address_fields)
-        return super(ShippingAddressView, self).form_valid(form)
+        return super().form_valid(form)
 
 
 class UserAddressUpdateView(CheckoutSessionMixin, generic.UpdateView):
     """
     Update a user address
     """
-    template_name = 'checkout/user_address_form.html'
+    template_name = 'oscar/checkout/user_address_form.html'
     form_class = UserAddressForm
     success_url = reverse_lazy('checkout:shipping-address')
 
@@ -206,20 +198,20 @@ class UserAddressUpdateView(CheckoutSessionMixin, generic.UpdateView):
         return self.request.user.addresses.all()
 
     def get_form_kwargs(self):
-        kwargs = super(UserAddressUpdateView, self).get_form_kwargs()
+        kwargs = super().get_form_kwargs()
         kwargs['user'] = self.request.user
         return kwargs
 
     def get_success_url(self):
         messages.info(self.request, _("Address saved"))
-        return super(UserAddressUpdateView, self).get_success_url()
+        return super().get_success_url()
 
 
 class UserAddressDeleteView(CheckoutSessionMixin, generic.DeleteView):
     """
     Delete an address from a user's address book.
     """
-    template_name = 'checkout/user_address_delete.html'
+    template_name = 'oscar/checkout/user_address_delete.html'
     success_url = reverse_lazy('checkout:shipping-address')
 
     def get_queryset(self):
@@ -227,7 +219,7 @@ class UserAddressDeleteView(CheckoutSessionMixin, generic.DeleteView):
 
     def get_success_url(self):
         messages.info(self.request, _("Address deleted"))
-        return super(UserAddressDeleteView, self).get_success_url()
+        return super().get_success_url()
 
 
 # ===============
@@ -247,15 +239,16 @@ class ShippingMethodView(CheckoutSessionMixin, generic.FormView):
     automatically selected.  Otherwise, a page is rendered where
     the user can choose the appropriate one.
     """
-    template_name = 'checkout/shipping_methods.html'
+    template_name = 'oscar/checkout/shipping_methods.html'
     form_class = ShippingMethodForm
     pre_conditions = ['check_basket_is_not_empty',
                       'check_basket_is_valid',
                       'check_user_email_is_captured']
+    success_url = reverse_lazy('checkout:payment-method')
 
     def post(self, request, *args, **kwargs):
         self._methods = self.get_available_shipping_methods()
-        return super(ShippingMethodView, self).post(request, *args, **kwargs)
+        return super().post(request, *args, **kwargs)
 
     def get(self, request, *args, **kwargs):
         # These pre-conditions can't easily be factored out into the normal
@@ -291,15 +284,15 @@ class ShippingMethodView(CheckoutSessionMixin, generic.FormView):
 
         # Must be more than one available shipping method, we present them to
         # the user to make a choice.
-        return super(ShippingMethodView, self).get(request, *args, **kwargs)
+        return super().get(request, *args, **kwargs)
 
     def get_context_data(self, **kwargs):
-        kwargs = super(ShippingMethodView, self).get_context_data(**kwargs)
+        kwargs = super().get_context_data(**kwargs)
         kwargs['methods'] = self._methods
         return kwargs
 
     def get_form_kwargs(self):
-        kwargs = super(ShippingMethodView, self).get_form_kwargs()
+        kwargs = super().get_form_kwargs()
         kwargs['methods'] = self._methods
         return kwargs
 
@@ -325,10 +318,10 @@ class ShippingMethodView(CheckoutSessionMixin, generic.FormView):
     def form_invalid(self, form):
         messages.error(self.request, _("Your submitted shipping method is not"
                                        " permitted"))
-        return super(ShippingMethodView, self).form_invalid(form)
+        return super().form_invalid(form)
 
     def get_success_response(self):
-        return redirect('checkout:payment-method')
+        return redirect(self.get_success_url())
 
 
 # ==============
@@ -350,6 +343,7 @@ class PaymentMethodView(CheckoutSessionMixin, generic.TemplateView):
         'check_user_email_is_captured',
         'check_shipping_data_is_captured']
     skip_conditions = ['skip_unless_payment_is_required']
+    success_url = reverse_lazy('checkout:payment-details')
 
     def get(self, request, *args, **kwargs):
         # By default we redirect straight onto the payment details view. Shops
@@ -358,7 +352,10 @@ class PaymentMethodView(CheckoutSessionMixin, generic.TemplateView):
         return self.get_success_response()
 
     def get_success_response(self):
-        return redirect('checkout:payment-details')
+        return redirect(self.get_success_url())
+
+    def get_success_url(self):
+        return str(self.success_url)
 
 
 # ================
@@ -375,7 +372,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
     being used. Chronologically, `payment-details` (preview=False) comes before
     `preview` (preview=True).
 
-    If sensitive details are required (eg a bankcard), then the payment details
+    If sensitive details are required (e.g. a bankcard), then the payment details
     view should submit to the preview URL and a custom implementation of
     `validate_payment_submission` should be provided.
 
@@ -398,8 +395,8 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
     All projects will need to subclass and customise this class as no payment
     is taken by default.
     """
-    template_name = 'checkout/payment_details.html'
-    template_name_preview = 'checkout/preview.html'
+    template_name = 'oscar/checkout/payment_details.html'
+    template_name_preview = 'oscar/checkout/preview.html'
 
     # These conditions are extended at runtime depending on whether we are in
     # 'preview' mode or not.
@@ -418,13 +415,13 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
             # The preview view needs to ensure payment information has been
             # correctly captured.
             return self.pre_conditions + ['check_payment_data_is_captured']
-        return super(PaymentDetailsView, self).get_pre_conditions(request)
+        return super().get_pre_conditions(request)
 
     def get_skip_conditions(self, request):
         if not self.preview:
             # Payment details should only be collected if necessary
             return ['skip_unless_payment_is_required']
-        return super(PaymentDetailsView, self).get_skip_conditions(request)
+        return super().get_skip_conditions(request)
 
     def post(self, request, *args, **kwargs):
         # Posting to payment-details isn't the right thing to do.  Form
@@ -516,7 +513,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
 
     def submit(self, user, basket, shipping_address, shipping_method,  # noqa (too complex (10))
                shipping_charge, billing_address, order_total,
-               payment_kwargs=None, order_kwargs=None):
+               payment_kwargs=None, order_kwargs=None, surcharges=None):
         """
         Submit a basket for order placement.
 
@@ -528,7 +525,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
            basket being manipulated during the payment process).
          * Attempt to take payment for the order
            - If payment is successful, place the order
-           - If a redirect is required (eg PayPal, 3DSecure), redirect
+           - If a redirect is required (e.g. PayPal, 3D Secure), redirect
            - If payment is unsuccessful, show an appropriate error message
 
         :basket: The basket to submit.
@@ -552,7 +549,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
         # We generate the order number first as this will be used
         # in payment requests (ie before the order model has been
         # created).  We also save it in the session for multi-stage
-        # checkouts (eg where we redirect to a 3rd party site and place
+        # checkouts (e.g. where we redirect to a 3rd party site and place
         # the order on a different request).
         order_number = self.generate_order_number(basket)
         self.checkout_session.set_order_number(order_number)
@@ -578,7 +575,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
         try:
             self.handle_payment(order_number, order_total, **payment_kwargs)
         except RedirectRequired as e:
-            # Redirect required (eg PayPal, 3DS)
+            # Redirect required (e.g. PayPal, 3DS)
             logger.info("Order #%s: redirecting to %s", order_number, e.url)
             return http.HttpResponseRedirect(e.url)
         except UnableToTakePayment as e:
@@ -586,14 +583,14 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
             # their bankcard has expired, wrong card number - that kind of
             # thing. This type of exception is supposed to set a friendly error
             # message that makes sense to the customer.
-            msg = six.text_type(e)
+            msg = str(e)
             logger.warning(
                 "Order #%s: unable to take payment (%s) - restoring basket",
                 order_number, msg)
             self.restore_frozen_basket()
 
             # We assume that the details submitted on the payment details view
-            # were invalid (eg expired bankcard).
+            # were invalid (e.g. expired bankcard).
             return self.render_payment_details(
                 self.request, error=msg, **payment_kwargs)
         except PaymentError as e:
@@ -603,7 +600,7 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
             # It makes sense to configure the checkout logger to
             # mail admins on an error as this issue warrants some further
             # investigation.
-            msg = six.text_type(e)
+            msg = str(e)
             logger.error("Order #%s: payment error (%s)", order_number, msg,
                          exc_info=True)
             self.restore_frozen_basket()
@@ -612,9 +609,9 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
         except Exception as e:
             # Unhandled exception - hopefully, you will only ever see this in
             # development...
-            logger.error(
+            logger.exception(
                 "Order #%s: unhandled exception while taking payment (%s)",
-                order_number, e, exc_info=True)
+                order_number, e)
             self.restore_frozen_basket()
             return self.render_preview(
                 self.request, error=error_msg, **payment_kwargs)
@@ -627,18 +624,24 @@ class PaymentDetailsView(OrderPlacementMixin, generic.TemplateView):
         try:
             return self.handle_order_placement(
                 order_number, user, basket, shipping_address, shipping_method,
-                shipping_charge, billing_address, order_total, **order_kwargs)
+                shipping_charge, billing_address, order_total, surcharges=surcharges, **order_kwargs)
         except UnableToPlaceOrder as e:
             # It's possible that something will go wrong while trying to
             # actually place an order.  Not a good situation to be in as a
             # payment transaction may already have taken place, but needs
             # to be handled gracefully.
-            msg = six.text_type(e)
+            msg = str(e)
             logger.error("Order #%s: unable to place order - %s",
                          order_number, msg, exc_info=True)
             self.restore_frozen_basket()
             return self.render_preview(
                 self.request, error=msg, **payment_kwargs)
+        except Exception as e:
+            # Hopefully you only ever reach this in development
+            logger.exception("Order #%s: unhandled exception while placing order (%s)", order_number, e)
+            error_msg = _("A problem occurred while placing this order. Please contact customer services.")
+            self.restore_frozen_basket()
+            return self.render_preview(self.request, error=error_msg, **payment_kwargs)
 
     def get_template_names(self):
         return [self.template_name_preview] if self.preview else [
@@ -654,31 +657,35 @@ class ThankYouView(generic.DetailView):
     """
     Displays the 'thank you' page which summarises the order just submitted.
     """
-    template_name = 'checkout/thank_you.html'
+    template_name = 'oscar/checkout/thank_you.html'
     context_object_name = 'order'
 
-    def get_object(self):
+    def get(self, request, *args, **kwargs):
+        self.object = self.get_object()
+        if self.object is None:
+            return redirect(settings.OSCAR_HOMEPAGE)
+        context = self.get_context_data(object=self.object)
+        return self.render_to_response(context)
+
+    def get_object(self, queryset=None):
         # We allow superusers to force an order thank-you page for testing
         order = None
         if self.request.user.is_superuser:
+            kwargs = {}
             if 'order_number' in self.request.GET:
-                order = Order._default_manager.get(
-                    number=self.request.GET['order_number'])
+                kwargs['number'] = self.request.GET['order_number']
             elif 'order_id' in self.request.GET:
-                order = Order._default_manager.get(
-                    id=self.request.GET['order_id'])
+                kwargs['id'] = self.request.GET['order_id']
+            order = Order._default_manager.filter(**kwargs).first()
 
         if not order:
             if 'checkout_order_id' in self.request.session:
-                order = Order._default_manager.get(
-                    pk=self.request.session['checkout_order_id'])
-            else:
-                raise http.Http404(_("No order found"))
-
+                order = Order._default_manager.filter(
+                    pk=self.request.session['checkout_order_id']).first()
         return order
 
     def get_context_data(self, *args, **kwargs):
-        ctx = super(ThankYouView, self).get_context_data(*args, **kwargs)
+        ctx = super().get_context_data(*args, **kwargs)
         # Remember whether this view has been loaded.
         # Only send tracking information on the first load.
         key = 'order_{}_thankyou_viewed'.format(ctx['order'].pk)
