@@ -6,7 +6,6 @@ from decimal import Decimal as D
 from django.conf import settings
 from django.core import exceptions
 from django.db import models
-from django.db.models import Exists, OuterRef
 from django.db.models.query import Q
 from django.template.defaultfilters import date as date_filter
 from django.urls import reverse
@@ -119,6 +118,13 @@ class AbstractConditionalOffer(models.Model):
         help_text=_("Exclusive offers cannot be combined on the same items"),
         default=True
     )
+    combinations = models.ManyToManyField(
+        'offer.ConditionalOffer',
+        help_text=_('Select other non-exclusive offers that this offer can be combined with on the same items'),
+        related_name='in_combination',
+        limit_choices_to={'exclusive': False},
+        blank=True,
+    )
 
     # We track a status variable so it's easier to load offers that are
     # 'available' in some sense.
@@ -336,8 +342,8 @@ class AbstractConditionalOffer(models.Model):
             .aggregate(total=models.Sum('frequency'))
         return aggregates['total'] if aggregates['total'] is not None else 0
 
-    def shipping_discount(self, charge):
-        return self.benefit.proxy().shipping_discount(charge)
+    def shipping_discount(self, charge, currency=None):
+        return self.benefit.proxy().shipping_discount(charge, currency)
 
     def record_usage(self, discount):
         self.num_applications += discount['freq']
@@ -443,6 +449,14 @@ class AbstractConditionalOffer(models.Model):
         return queryset.filter(is_discountable=True).exclude(
             structure=Product.CHILD)
 
+    @cached_property
+    def combined_offers(self):
+        return self.__class__.objects.filter(
+            models.Q(pk=self.pk)
+            | models.Q(pk__in=self.combinations.values_list("pk", flat=True))
+            | models.Q(pk__in=self.in_combination.values_list("pk", flat=True))
+        ).distinct()
+
 
 class AbstractBenefit(BaseOfferMixin, models.Model):
     range = models.ForeignKey(
@@ -615,14 +629,14 @@ class AbstractBenefit(BaseOfferMixin, models.Model):
         if errors:
             raise exceptions.ValidationError(errors)
 
-    def round(self, amount):
+    def round(self, amount, currency=None):
         """
         Apply rounding to discount amount
         """
         rounding_function_path = getattr(settings, 'OSCAR_OFFER_ROUNDING_FUNCTION', None)
         if rounding_function_path:
             rounding_function = cached_import_string(rounding_function_path)
-            return rounding_function(amount)
+            return rounding_function(amount, currency)
 
         return amount.quantize(D('.01'), ROUND_DOWN)
 
@@ -665,7 +679,7 @@ class AbstractBenefit(BaseOfferMixin, models.Model):
         # We sort lines to be cheapest first to ensure consistent applications
         return sorted(line_tuples, key=operator.itemgetter(0))
 
-    def shipping_discount(self, charge):
+    def shipping_discount(self, charge, currency=None):
         return D('0.00')
 
 
@@ -810,6 +824,7 @@ class AbstractRange(models.Model):
     class Meta:
         abstract = True
         app_label = 'offer'
+        ordering = ['name']
         verbose_name = _("Range")
         verbose_name_plural = _("Ranges")
 
@@ -911,29 +926,32 @@ class AbstractRange(models.Model):
                 id__in=self.excluded_products.values("id")
             )
 
-        Category = self.included_categories.model
+        if self.included_categories.exists():
+            # build query to select all category subtrees.
+            category_filter = Q()
+            for path, depth in self.included_categories.values_list("path", "depth"):
+                category_filter |= Q(
+                    categories__depth__gte=depth, categories__path__startswith=path
+                )
 
-        # build query to select all category subtrees.
-        included_in_subtree = self.included_categories.filter(
-            path__rstartswith=OuterRef("path"), depth__lte=OuterRef("depth")
-        )
-        category_tree = Category.objects.annotate(
-            is_included_in_subtree=Exists(included_in_subtree.values("id"))
-        ).filter(is_included_in_subtree=True)
-
-        # select all those product that are selected either by product class,
-        # category, or explicitly by included_products.
-        selected_parents = (
-            Product.objects.filter(
+            # select all those product that are selected either by product class,
+            # category, or explicitly by included_products.
+            selected_parents = Product.objects.annotate(
+                selected_categories=models.FilteredRelation(
+                    "categories", condition=category_filter
+                )
+            ).filter(
                 Q(product_class_id__in=self.classes.values("id"))
-                | Q(categories__in=category_tree)
-            )
-            | self.included_products.all()
-        )
+                | Q(selected_categories__isnull=False)
+            ) | self.included_products.all()
+        else:
+            selected_parents = Product.objects.filter(
+                product_class_id__in=self.classes.values("id")
+            ) | self.included_products.all()
 
         # select parents and their children
-        selected_products = (
-            selected_parents | Product.objects.filter(parent__in=selected_parents)
+        selected_products = selected_parents | Product.objects.filter(
+            parent__in=selected_parents
         )
 
         # now go and exclude all explicitly excluded products
