@@ -1,18 +1,15 @@
-import hashlib
 import logging
 from collections import OrderedDict
 from decimal import Decimal as D
 
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
 from django.core.signing import BadSignature, Signer
 from django.db import models
 from django.db.models import Sum
 from django.utils import timezone
 from django.utils.crypto import constant_time_compare
-from django.utils.encoding import python_2_unicode_compatible
 from django.utils.timezone import now
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.utils.translation import pgettext_lazy
 
 from oscar.apps.order.signals import (
@@ -24,11 +21,9 @@ from oscar.models.fields import AutoSlugField
 
 from . import exceptions
 
-
 logger = logging.getLogger('oscar.order')
 
 
-@python_2_unicode_compatible
 class AbstractOrder(models.Model):
     """
     The main order model
@@ -151,8 +146,14 @@ class AbstractOrder(models.Model):
                                   new_status=new_status,
                                   )
 
+        self._create_order_status_change(old_status, new_status)
+
     set_status.alters_data = True
 
+    def _create_order_status_change(self, old_status, new_status):
+        # Not setting the status on the order as that should be handled before
+        self.status_changes.create(old_status=old_status, new_status=new_status)
+
     @property
     def is_anonymous(self):
         # It's possible for an order to be placed by a customer who then
@@ -165,44 +166,40 @@ class AbstractOrder(models.Model):
         """
         Return basket total including tax but before discounts are applied
         """
-        total = D('0.00')
-        for line in self.lines.all():
-            total += line.line_price_before_discounts_incl_tax
-        return total
+        result = self.lines.aggregate(total=Sum('line_price_before_discounts_incl_tax'))
+        return result['total']
 
     @property
     def basket_total_before_discounts_excl_tax(self):
         """
         Return basket total excluding tax but before discounts are applied
         """
-        total = D('0.00')
-        for line in self.lines.all():
-            total += line.line_price_before_discounts_excl_tax
-        return total
+        result = self.lines.aggregate(total=Sum('line_price_before_discounts_excl_tax'))
+        return result['total']
 
     @property
     def basket_total_incl_tax(self):
         """
         Return basket total including tax
         """
-        return self.total_incl_tax - self.shipping_incl_tax
+        return self.total_incl_tax - self.shipping_incl_tax - self.surcharge_incl_tax
 
     @property
     def basket_total_excl_tax(self):
         """
         Return basket total excluding tax
         """
-        return self.total_excl_tax - self.shipping_excl_tax
+        return self.total_excl_tax - self.shipping_excl_tax - self.surcharge_excl_tax
 
     @property
     def total_before_discounts_incl_tax(self):
-        return (self.basket_total_before_discounts_incl_tax +
-                self.shipping_incl_tax)
+        return (self.basket_total_before_discounts_incl_tax
+                + self.shipping_incl_tax)
 
     @property
     def total_before_discounts_excl_tax(self):
-        return (self.basket_total_before_discounts_excl_tax +
-                self.shipping_excl_tax)
+        return (self.basket_total_before_discounts_excl_tax
+                + self.shipping_excl_tax)
 
     @property
     def total_discount_incl_tax(self):
@@ -225,6 +222,14 @@ class AbstractOrder(models.Model):
     def total_tax(self):
         return self.total_incl_tax - self.total_excl_tax
 
+    @property
+    def surcharge_excl_tax(self):
+        return sum(charge.excl_tax for charge in self.surcharges.all())
+
+    @property
+    def surcharge_incl_tax(self):
+        return sum(charge.incl_tax for charge in self.surcharges.all())
+
     @property
     def num_lines(self):
         return self.lines.count()
@@ -303,44 +308,17 @@ class AbstractOrder(models.Model):
         verbose_name_plural = _("Orders")
 
     def __str__(self):
-        return u"#%s" % (self.number,)
+        return "#%s" % (self.number,)
 
     def verification_hash(self):
         signer = Signer(salt='oscar.apps.order.Order')
         return signer.sign(self.number)
 
-    def check_deprecated_verification_hash(self, hash_to_check):
-        """
-        Backward compatible check for md5 hashes that were generated in
-        Oscar 1.5 and lower.
-
-        This must explicitly be enabled by setting OSCAR_DEPRECATED_ORDER_VERIFY_KEY,
-        which must not be equal to SECRET_KEY - i.e., the project must
-        have changed its SECRET_KEY since this change was applied.
-
-        TODO: deprecate this method in Oscar 2.0, and remove it in Oscar 2.1.
-        """
-        old_verification_key = getattr(settings, 'OSCAR_DEPRECATED_ORDER_VERIFY_KEY', None)
-        if old_verification_key is None:
-            return False
-
-        if old_verification_key == settings.SECRET_KEY:
-            raise ImproperlyConfigured(
-                'OSCAR_DEPRECATED_ORDER_VERIFY_KEY cannot be equal to SECRET_KEY')
-
-        logger.warning('Using insecure md5 hashing for order URL hash verification.')
-        string_to_hash = '%s%s' % (self.number, old_verification_key)
-        order_hash = hashlib.md5(string_to_hash.encode('utf8')).hexdigest()
-        return constant_time_compare(order_hash, hash_to_check)
-
     def check_verification_hash(self, hash_to_check):
         """
         Checks the received verification hash against this order number.
         Returns False if the verification failed, True otherwise.
         """
-        if self.check_deprecated_verification_hash(hash_to_check):
-            return True
-
         signer = Signer(salt='oscar.apps.order.Order')
         try:
             signed_number = signer.unsign(hash_to_check)
@@ -381,10 +359,9 @@ class AbstractOrder(models.Model):
         # this gives us the ability to set the date_placed explicitly (which is
         # useful when importing orders from another system).
         self.set_date_placed_default()
-        super(AbstractOrder, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
 
 
-@python_2_unicode_compatible
 class AbstractOrderNote(models.Model):
     """
     A note against an order.
@@ -420,11 +397,12 @@ class AbstractOrderNote(models.Model):
     class Meta:
         abstract = True
         app_label = 'order'
+        ordering = ['-date_updated']
         verbose_name = _("Order Note")
         verbose_name_plural = _("Order Notes")
 
     def __str__(self):
-        return u"'%s' (%s)" % (self.message[0:50], self.user)
+        return "'%s' (%s)" % (self.message[0:50], self.user)
 
     def is_editable(self):
         if self.note_type == self.SYSTEM:
@@ -433,7 +411,29 @@ class AbstractOrderNote(models.Model):
         return delta.seconds < self.editable_lifetime
 
 
-@python_2_unicode_compatible
+class AbstractOrderStatusChange(models.Model):
+    order = models.ForeignKey(
+        'order.Order',
+        on_delete=models.CASCADE,
+        related_name='status_changes',
+        verbose_name=_('Order Status Changes')
+    )
+    old_status = models.CharField(_('Old Status'), max_length=100, blank=True)
+    new_status = models.CharField(_('New Status'), max_length=100, blank=True)
+    date_created = models.DateTimeField(_('Date Created'), auto_now_add=True, db_index=True)
+
+    class Meta:
+        abstract = True
+        app_label = 'order'
+        verbose_name = _('Order Status Change')
+        verbose_name_plural = _('Order Status Changes')
+        ordering = ['-date_created']
+
+    def __str__(self):
+        return _("%(order)s has changed status from %(old_status)s to %(new_status)s") \
+            % {'order': self.order, 'old_status': self.old_status, 'new_status': self.new_status, }
+
+
 class AbstractCommunicationEvent(models.Model):
     """
     An order-level event involving a communication to the customer, such
@@ -445,10 +445,10 @@ class AbstractCommunicationEvent(models.Model):
         related_name="communication_events",
         verbose_name=_("Order"))
     event_type = models.ForeignKey(
-        'customer.CommunicationEventType',
+        'communication.CommunicationEventType',
         on_delete=models.CASCADE,
         verbose_name=_("Event Type"))
-    date_created = models.DateTimeField(_("Date"), auto_now_add=True)
+    date_created = models.DateTimeField(_("Date"), auto_now_add=True, db_index=True)
 
     class Meta:
         abstract = True
@@ -465,7 +465,6 @@ class AbstractCommunicationEvent(models.Model):
 # LINES
 
 
-@python_2_unicode_compatible
 class AbstractLine(models.Model):
     """
     An order line
@@ -514,7 +513,7 @@ class AbstractLine(models.Model):
         'catalogue.Product', on_delete=models.SET_NULL, blank=True, null=True,
         verbose_name=_("Product"))
     title = models.CharField(
-        pgettext_lazy(u"Product title", u"Title"), max_length=255)
+        pgettext_lazy("Product title", "Title"), max_length=255)
     # UPC can be null because it's usually set as the product's UPC, and that
     # can be null as well
     upc = models.CharField(_("UPC"), max_length=128, blank=True, null=True)
@@ -526,10 +525,10 @@ class AbstractLine(models.Model):
 
     # Price information (these fields are actually redundant as the information
     # can be calculated from the LinePrice models
-    # Deprecated - will be removed in Oscar 2.0
+    # Deprecated - will be removed in Oscar 2.1
     line_price_incl_tax = models.DecimalField(
         _("Price (inc. tax)"), decimal_places=2, max_digits=12)
-    # Deprecated - will be removed in Oscar 2.0
+    # Deprecated - will be removed in Oscar 2.1
     line_price_excl_tax = models.DecimalField(
         _("Price (excl. tax)"), decimal_places=2, max_digits=12)
 
@@ -541,10 +540,6 @@ class AbstractLine(models.Model):
         _("Price before discounts (excl. tax)"),
         decimal_places=2, max_digits=12)
 
-    # Deprecated - will be removed in Oscar 2.0
-    unit_cost_price = models.DecimalField(
-        _("Unit Cost Price"), decimal_places=2, max_digits=12, blank=True,
-        null=True)
     # Normal site price for item (without discounts)
     unit_price_incl_tax = models.DecimalField(
         _("Unit Price (inc. tax)"), decimal_places=2, max_digits=12,
@@ -552,19 +547,11 @@ class AbstractLine(models.Model):
     unit_price_excl_tax = models.DecimalField(
         _("Unit Price (excl. tax)"), decimal_places=2, max_digits=12,
         blank=True, null=True)
-    # Deprecated - will be removed in Oscar 2.0
-    unit_retail_price = models.DecimalField(
-        _("Unit Retail Price"), decimal_places=2, max_digits=12,
-        blank=True, null=True)
 
     # Partners often want to assign some status to each line to help with their
     # own business processes.
     status = models.CharField(_("Status"), max_length=255, blank=True)
 
-    # Deprecated - will be removed in Oscar 2.0
-    est_dispatch_date = models.DateField(
-        _("Estimated Dispatch Date"), blank=True, null=True)
-
     #: Order status pipeline.  This should be a dict where each (key, value)
     #: corresponds to a status and the possible statuses that can follow that
     #: one.
@@ -691,7 +678,7 @@ class AbstractLine(models.Model):
         """
         Test whether a shipping event with the given quantity is permitted
 
-        This method should normally be overriden to ensure that the
+        This method should normally be overridden to ensure that the
         prerequisite shipping events have been passed for this line.
         """
         # Note, this calculation is simplistic - normally, you will also need
@@ -800,7 +787,6 @@ class AbstractLine(models.Model):
         return True, None
 
 
-@python_2_unicode_compatible
 class AbstractLineAttribute(models.Model):
     """
     An attribute of a line
@@ -826,7 +812,6 @@ class AbstractLineAttribute(models.Model):
         return "%s = %s" % (self.type, self.value)
 
 
-@python_2_unicode_compatible
 class AbstractLinePrice(models.Model):
     """
     For tracking the prices paid for each unit within a line.
@@ -872,7 +857,6 @@ class AbstractLinePrice(models.Model):
 # PAYMENT EVENTS
 
 
-@python_2_unicode_compatible
 class AbstractPaymentEventType(models.Model):
     """
     Payment event types are things like 'Paid', 'Failed', 'Refunded'.
@@ -894,7 +878,6 @@ class AbstractPaymentEventType(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class AbstractPaymentEvent(models.Model):
     """
     A payment event for an order
@@ -930,7 +913,7 @@ class AbstractPaymentEvent(models.Model):
         null=True,
         on_delete=models.CASCADE,
         related_name='payment_events')
-    date_created = models.DateTimeField(_("Date created"), auto_now_add=True)
+    date_created = models.DateTimeField(_("Date created"), auto_now_add=True, db_index=True)
 
     class Meta:
         abstract = True
@@ -972,7 +955,6 @@ class PaymentEventQuantity(models.Model):
 # SHIPPING EVENTS
 
 
-@python_2_unicode_compatible
 class AbstractShippingEvent(models.Model):
     """
     An event is something which happens to a group of lines such as
@@ -994,7 +976,7 @@ class AbstractShippingEvent(models.Model):
         _("Event notes"), blank=True,
         help_text=_("This could be the dispatch reference, or a "
                     "tracking number"))
-    date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
+    date_created = models.DateTimeField(_("Date Created"), auto_now_add=True, db_index=True)
 
     class Meta:
         abstract = True
@@ -1012,7 +994,6 @@ class AbstractShippingEvent(models.Model):
         return self.lines.count()
 
 
-@python_2_unicode_compatible
 class ShippingEventQuantity(models.Model):
     """
     A "through" model linking lines to shipping events.
@@ -1046,7 +1027,7 @@ class ShippingEventQuantity(models.Model):
         if not self.line.is_shipping_event_permitted(
                 self.event.event_type, self.quantity):
             raise exceptions.InvalidShippingEvent
-        super(ShippingEventQuantity, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
 
     def __str__(self):
         return _("%(product)s - quantity %(qty)d") % {
@@ -1054,12 +1035,11 @@ class ShippingEventQuantity(models.Model):
             'qty': self.quantity}
 
 
-@python_2_unicode_compatible
 class AbstractShippingEventType(models.Model):
     """
-    A type of shipping/fulfillment event
+    A type of shipping/fulfilment event
 
-    Eg: 'Shipped', 'Cancelled', 'Returned'
+    E.g.: 'Shipped', 'Cancelled', 'Returned'
     """
     # Name is the friendly description of an event
     name = models.CharField(_("Name"), max_length=255, unique=True)
@@ -1081,7 +1061,6 @@ class AbstractShippingEventType(models.Model):
 # DISCOUNTS
 
 
-@python_2_unicode_compatible
 class AbstractOrderDiscount(models.Model):
     """
     A discount against an order.
@@ -1142,6 +1121,7 @@ class AbstractOrderDiscount(models.Model):
     class Meta:
         abstract = True
         app_label = 'order'
+        ordering = ['pk']
         verbose_name = _("Order Discount")
         verbose_name_plural = _("Order Discounts")
 
@@ -1156,7 +1136,7 @@ class AbstractOrderDiscount(models.Model):
             if voucher:
                 self.voucher_code = voucher.code
 
-        super(AbstractOrderDiscount, self).save(**kwargs)
+        super().save(**kwargs)
 
     def __str__(self):
         return _("Discount of %(amount)r from order %(order)s") % {
@@ -1181,4 +1161,37 @@ class AbstractOrderDiscount(models.Model):
     def description(self):
         if self.voucher_code:
             return self.voucher_code
-        return self.offer_name or u""
+        return self.offer_name or ""
+
+
+class AbstractSurcharge(models.Model):
+    order = models.ForeignKey(
+        'order.Order',
+        on_delete=models.CASCADE,
+        related_name="surcharges",
+        verbose_name=_("Surcharges"))
+
+    name = models.CharField(
+        _("Surcharge"), max_length=128
+    )
+
+    code = models.CharField(
+        _("Surcharge code"), max_length=128
+    )
+
+    incl_tax = models.DecimalField(
+        _("Surcharge (inc. tax)"), decimal_places=2, max_digits=12,
+        default=0)
+
+    excl_tax = models.DecimalField(
+        _("Surcharge (excl. tax)"), decimal_places=2, max_digits=12,
+        default=0)
+
+    @property
+    def tax(self):
+        return self.incl_tax - self.excl_tax
+
+    class Meta:
+        abstract = True
+        app_label = 'order'
+        ordering = ['pk']
