@@ -74,12 +74,23 @@ var oscar = (function(o, $) {
         init: function(options) {
             // Run initialisation that should take place on every page of the dashboard.
             var defaults = {
-                'languageCode': 'en',
-                'dateFormat': 'yy-mm-dd',
-                'timeFormat': 'hh:ii',
-                'datetimeFormat': 'yy-mm-dd hh:ii',
+                'dateFormat': 'DD/MM/YYYY',
+                'timeFormat': 'HH:mm',
+                'datetimeFormat': 'DD/MM/YYYY HH:mm',
                 'stepMinute': 15,
-                'initialDate': new Date(new Date().setSeconds(0)),
+                'datetimePickerConfig': {
+                    icons: {
+                        time: 'fas fa-clock',
+                        date: 'fas fa-calendar',
+                        up: 'fas fa-arrow-up',
+                        down: 'fas fa-arrow-down',
+                        previous: 'fas fa-chevron-left',
+                        next: 'fas fa-chevron-right',
+                        today: 'fas fa-calendar-check-o',
+                        clear: 'fas fa-trash',
+                        close: 'fas fa-times'
+                    }
+                },
                 'tinyConfig': {
                     entity_encoding: 'raw',
                     statusbar: false,
@@ -110,9 +121,9 @@ var oscar = (function(o, $) {
             });
 
             // Adds error icon if there are errors in the product update form
-            $('[data-behaviour="affix-nav-errors"] .tab-pane').each(function(){
+            $('[data-behaviour="tab-nav-errors"] .tab-pane').each(function(){
                 var productErrorListener = $(this).find('[class*="error"]:not(:empty)').closest('.tab-pane').attr('id');
-                $('[data-spy="affix"] a[href="#' + productErrorListener + '"]').append('<i class="icon-info-sign pull-right"></i>');
+                $('.tab-nav a[href="#' + productErrorListener + '"]').append('<i class="fas fa-info-circle float-right"></i>');
             });
 
             o.dashboard.filereader.init();
@@ -205,11 +216,14 @@ var oscar = (function(o, $) {
         },
         initDatePickers: function(el) {
             if ($.fn.datetimepicker) {
+
+                // Set "Tempus Dominus Bootstrap 4" datetime picker's global options.
+                $.fn.datetimepicker.Constructor.Default = $.extend(
+                    {}, $.fn.datetimepicker.Constructor.Default, o.dashboard.options.datetimePickerConfig
+                );
+
                 var defaultDatepickerConfig = {
                     'format': o.dashboard.options.dateFormat,
-                    'autoclose': true,
-                    'language': o.dashboard.options.languageCode,
-                    'minView': 2
                 };
                 var $dates = $(el).find('[data-oscarWidget="date"]').not('.no-widget-init').not('.no-widget-init *');
                 $dates.each(function(ind, ele) {
@@ -222,37 +236,29 @@ var oscar = (function(o, $) {
 
                 var defaultDatetimepickerConfig = {
                     'format': o.dashboard.options.datetimeFormat,
-                    'minuteStep': o.dashboard.options.stepMinute,
-                    'autoclose': true,
-                    'language': o.dashboard.options.languageCode,
-                    'initialDate': o.dashboard.options.initialDate
+                    'stepping': o.dashboard.options.stepMinute
                 };
                 var $datetimes = $(el).find('[data-oscarWidget="datetime"]').not('.no-widget-init').not('.no-widget-init *');
                 $datetimes.each(function(ind, ele) {
                     var $ele = $(ele),
                         config = $.extend({}, defaultDatetimepickerConfig, {
                             'format': $ele.data('datetimeformat'),
-                            'minuteStep': $ele.data('stepminute')
+                            'stepping': $ele.data('stepminute')
                         });
                     $ele.datetimepicker(config);
                 });
 
                 var defaultTimepickerConfig = {
                     'format': o.dashboard.options.timeFormat,
-                    'minuteStep': o.dashboard.options.stepMinute,
-                    'autoclose': true,
-                    'language': o.dashboard.options.languageCode,
-                    'initialDate': o.dashboard.options.initialDate
+                    'stepping': o.dashboard.options.stepMinute
                 };
                 var $times = $(el).find('[data-oscarWidget="time"]').not('.no-widget-init').not('.no-widget-init *');
                 $times.each(function(ind, ele) {
                     var $ele = $(ele),
                         config = $.extend({}, defaultTimepickerConfig, {
                             'format': $ele.data('timeformat'),
-                            'minuteStep': $ele.data('stepminute'),
-                            'startView': 1,
-                            'maxView': 1,
-                            'formatViewType': 'time'
+                            'stepping': $ele.data('stepminute'),
+                            'viewMode': 'times'
                         });
                     $ele.datetimepicker(config);
                 });
@@ -266,14 +272,22 @@ var oscar = (function(o, $) {
         },
         initForms: function() {
             // Disable buttons when they are clicked and show a "loading" message taken from the
-            // data-loading-text attribute (http://getbootstrap.com/2.3.2/javascript.html#buttons).
+            // data-loading-text attribute.
             // Do not disable if button is inside a form with invalid fields.
             // This uses a delegated event so that it keeps working for forms that are reloaded
             // via AJAX: https://api.jquery.com/on/#direct-and-delegated-events
             $(document.body).on('click', '[data-loading-text]', function(){
-                var form = $(this).parents("form");
-                if (!form || $(":invalid", form).length == 0)
-                    $(this).button('loading');
+                var $btn_or_input = $(this),
+                    form = $btn_or_input.parents("form");
+                if (!form || $(":invalid", form).length == 0) {
+                    var d = 'disabled',
+                        val = $btn_or_input.is('input') ? 'val' : 'html';
+                    // push to event loop so as not to delay form submission
+                    setTimeout(function() {
+                        $btn_or_input[val]($btn_or_input.data('loading-text'));
+                        $btn_or_input.addClass(d).attr(d, d).prop(d, true);
+                    });
+                }
             });
 
             // Display tabs that have invalid input fields
@@ -315,7 +329,7 @@ var oscar = (function(o, $) {
             },
             adjustBenefitForm: function() {
                 var type = $('#id_type').val(),
-                    $valueContainer = $('#id_value').parents('.control-group');
+                    $valueContainer = $('#id_value').parents('.form-group');
                 if (type == 'Multibuy') {
                     $('#id_value').val('');
                     $valueContainer.hide();
@@ -340,7 +354,13 @@ var oscar = (function(o, $) {
             toggleOptionGroup: function(type_select){
                 var option_group_select = $('#' + type_select.attr('id').replace('type', 'option_group'));
                 var v = type_select.val();
-                option_group_select.parent().parent().toggle(v === 'option' || v === 'multi_option');
+                var showOptionGroup = v === 'option' || v === 'multi_option';
+                option_group_select.parent().parent().toggle(showOptionGroup);
+                if(showOptionGroup){
+                    option_group_select.attr('required', 'required');
+                }else{
+                    option_group_select.attr('required', null);
+                }
             }
         },
         ranges: {
@@ -423,24 +443,6 @@ var oscar = (function(o, $) {
                 saveOrder: saveOrder
             };
         }()),
-        search: {
-            init: function() {
-                var searchForm = $(".orders_search"),
-                    searchLink = $('.pull_out'),
-                    doc = $('document');
-                searchForm.each(function() {
-                    doc.css('height', doc.height());
-                });
-                searchLink.on('click', function() {
-                    searchForm.parent()
-                        .find('.pull-left')
-                        .toggleClass('no-float')
-                        .end().end()
-                        .slideToggle("fast");
-                }
-                );
-            }
-        },
         filereader: {
             init: function () {
                 // Add local file loader to update image files on change in
